"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Cerbos = exports.AuthorizationError = exports.AuthorizeEffect = void 0;
const node_fetch_1 = __importDefault(require("node-fetch"));
const uuid_1 = require("uuid");
const winston = __importStar(require("winston"));
var AuthorizeEffect;
(function (AuthorizeEffect) {
    AuthorizeEffect["ALLOW"] = "EFFECT_ALLOW";
    AuthorizeEffect["DENY"] = "EFFECT_DENY";
})(AuthorizeEffect = exports.AuthorizeEffect || (exports.AuthorizeEffect = {}));
class AuthorizationError extends Error {
}
exports.AuthorizationError = AuthorizationError;
class CerbosResponseWrapper {
    constructor(resp) {
        this.resp = resp;
    }
    isAuthorized(resourceKey, action) {
        var _a;
        try {
            const allowed = ((_a = this.resp.resourceInstances[resourceKey]) === null || _a === void 0 ? void 0 : _a.actions[action]) ==
                AuthorizeEffect.ALLOW;
            return allowed !== null && allowed !== void 0 ? allowed : false;
        }
        catch (e) {
            return false;
        }
    }
}
class Cerbos {
    constructor({ hostname, logLevel, timeout = 0, playgroundInstance, }) {
        this.host = hostname;
        this.timeout = timeout;
        this.playgroundInstance = playgroundInstance;
        this.log = winston.createLogger({
            level: logLevel,
            silent: !logLevel,
            transports: [
                new winston.transports.Console({
                    format: winston.format.simple(),
                }),
            ],
        });
    }
    check(data) {
        return __awaiter(this, void 0, void 0, function* () {
            this.log.info("Cerbos.check", data);
            const payload = Object.assign(Object.assign({ requestId: (0, uuid_1.v4)() }, data), { resource: Object.assign({ policyVersion: data.resource.policyVersion || "default" }, data.resource), principal: Object.assign({ policyVersion: data.principal.policyVersion || "default" }, data.principal) });
            this.log.debug("Cerbos.check Payload", payload);
            // eslint-disable-next-line prefer-const
            // eslint-disable-next-line @typescript-eslint/no-explicit-any
            let headers = {
                "Content-Type": "application/json",
            };
            if (this.playgroundInstance) {
                headers = Object.assign(Object.assign({}, headers), { "Playground-Instance": this.playgroundInstance });
            }
            try {
                const response = yield (0, node_fetch_1.default)(`${this.host}/api/check`, {
                    method: "post",
                    body: JSON.stringify(payload),
                    headers,
                    timeout: this.timeout,
                });
                const data = yield response.json();
                this.log.info("Cerbos.check: Response", data);
                return new CerbosResponseWrapper(data);
            }
            catch (e) {
                this.log.error("Cerbos.check Error", e);
                throw new AuthorizationError(`Could not connect to Cerbos PDP at ${this.host}`);
            }
        });
    }
}
exports.Cerbos = Cerbos;
exports.default = {
    Cerbos,
};
