var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import fetch from "node-fetch";
import { v4 as uuidv4 } from "uuid";
import * as winston from "winston";
export var AuthorizeEffect;
(function (AuthorizeEffect) {
    AuthorizeEffect["ALLOW"] = "EFFECT_ALLOW";
    AuthorizeEffect["DENY"] = "EFFECT_DENY";
})(AuthorizeEffect || (AuthorizeEffect = {}));
export class AuthorizationError extends Error {
}
class CerbosResponseWrapper {
    constructor(resp) {
        this.resp = resp;
    }
    isAuthorized(resourceKey, action) {
        var _a;
        const allowed = ((_a = this.resp.resourceInstances[resourceKey]) === null || _a === void 0 ? void 0 : _a.actions[action]) ==
            AuthorizeEffect.ALLOW;
        return allowed !== null && allowed !== void 0 ? allowed : false;
    }
}
export default class Cerbos {
    constructor({ hostname, logLevel, timeout = 0 }) {
        this.host = hostname;
        this.timeout = timeout;
        this.log = winston.createLogger({
            level: logLevel,
            silent: !logLevel,
            transports: [
                new winston.transports.Console({
                    format: winston.format.simple(),
                }),
            ],
        });
    }
    check(data) {
        return __awaiter(this, void 0, void 0, function* () {
            this.log.info("Cerbos.check", data);
            const payload = Object.assign(Object.assign({ requestId: uuidv4() }, data), { resource: Object.assign({ policyVersion: data.resource.policyVersion || "default" }, data.resource), principal: Object.assign({ policyVersion: data.principal.policyVersion || "default" }, data.principal) });
            this.log.debug("Cerbos.check Payload", payload);
            try {
                const response = yield fetch(`${this.host}/api/check`, {
                    method: "post",
                    body: JSON.stringify(payload),
                    headers: { "Content-Type": "application/json" },
                    timeout: this.timeout,
                });
                const data = yield response.json();
                this.log.info("Cerbos.check: Response", data);
                return new CerbosResponseWrapper(data);
            }
            catch (e) {
                this.log.error("Cerbos.check Error", e);
                throw new AuthorizationError(`Could not connect to Cerbos PDP at ${this.host}`);
            }
        });
    }
}
