interface IPrincipal {
    id: string;
    policyVersion?: any;
    roles: string[];
    attr?: {
        [key: string]: any;
    };
}
interface IAuxData {
    jwt?: IJwt;
}
interface IJwt {
    token: string;
    keySetId?: string;
}
interface IAuthorize {
    actions: string[];
    resource: {
        policyVersion?: any;
        kind: string;
        instances: {
            [resourceKey: string]: {
                attr?: {
                    [key: string]: any;
                };
            };
        };
    };
    principal: IPrincipal;
    auxData?: IAuxData;
}
export interface ICerbosBatchAuthorizeResource {
    actions: string[];
    resource: {
        policyVersion?: any;
        kind: string;
        id: string;
        attr: {
            [key: string]: any;
        };
    };
}
export interface ICerbosBatchAuthorizeResult {
    [key: string]: AuthorizeEffect;
}
export declare enum AuthorizeEffect {
    ALLOW = "EFFECT_ALLOW",
    DENY = "EFFECT_DENY"
}
export declare class AuthorizationError extends Error {
}
export interface ICerbosResponse {
    isAuthorized: (resourceKey: string, action: string) => boolean;
}
interface CerbosOptions {
    hostname: string;
    logLevel?: "fatal" | "error" | "warn" | "info" | "debug";
    timeout?: number;
    playgroundInstance?: string;
}
export declare class Cerbos {
    private host;
    private log;
    private timeout;
    private playgroundInstance?;
    constructor({ hostname, logLevel, timeout, playgroundInstance, }: CerbosOptions);
    check(data: IAuthorize): Promise<ICerbosResponse>;
}
declare const _default: {
    Cerbos: typeof Cerbos;
};
export default _default;
